#ifndef GOOGLE_PROTOBUF_CPP_EDITION_DEFAULTS_H__
#define GOOGLE_PROTOBUF_CPP_EDITION_DEFAULTS_H__

// This file contains the serialized FeatureSetDefaults object corresponding to
// the C++ runtime.  This is used for feature resolution under Editions.
// NOLINTBEGIN
// clang-format off
#define PROTOBUF_INTERNAL_CPP_EDITION_DEFAULTS "\n\026\022\021\010\001\020\002\030\002 \003(\0010\002\302>\002\010\001\030\346\007\n\026\022\021\010\002\020\001\030\001 \002(\0010\001\302>\002\010\000\030\347\007\n\026\022\021\010\001\020\001\030\001 \002(\0010\001\302>\002\010\000\030\350\007 \346\007(\350\007"
// clang-format on
// NOLINTEND

#endif  // GOOGLE_PROTOBUF_CPP_EDITION_DEFAULTS_H__
